@@ -16,6 +16,7 @@
 #include <libbonobo.h>
 
 #include <gconf/gconf-client.h>
+#include <gdk/gdkx.h>
 
 #include "cddb-slave.h"
 
@@ -26,6 +27,8 @@
 static int running_objects = 0;
 static GConfClient *client = NULL;
 
+gboolean cddb_debugging = FALSE;
+
 enum {
 	CDDB_SEND_FAKE_INFO,
 	CDDB_SEND_REAL_INFO,
@@ -76,12 +79,12 @@ factory_fn (BonoboGenericFactory *factory,
 	info = gconf_client_get_int (client, "/apps/CDDB-Slave2/info", NULL);
 	switch (info) {
 	case CDDB_SEND_FAKE_INFO:
-		name = g_strdup ("cddbslave2-user");
-		hostname = g_strdup ("192.168.77.77");
+		name = g_strdup ("unknown");
+		hostname = g_strdup ("localhost");
 		break;
 
 	case CDDB_SEND_REAL_INFO:
-		name = g_get_user_name ();
+		name = g_strdup (g_get_user_name ());
 		hostname = get_hostname ();
 		break;
 
@@ -122,15 +125,23 @@ static gboolean
 cddbslave_init (gpointer data)
 {
 	BonoboGenericFactory *factory;
+	char *display_iid;
 
-	factory = bonobo_generic_factory_new (CDDBSLAVE_IID, factory_fn, NULL);
-	if (factory == NULL) {
-		g_error ("Cannot create factory");
-		exit (1);
+	factory = (BonoboGenericFactory*)bonobo_activation_activate_from_id (CDDBSLAVE_IID, Bonobo_ACTIVATION_FLAG_EXISTING_ONLY, NULL, NULL);
+
+	if (!factory) {
+		display_iid = bonobo_activation_make_registration_id (CDDBSLAVE_IID, DisplayString (gdk_display));
+		factory = bonobo_generic_factory_new (display_iid, factory_fn, NULL);
+		g_free (display_iid);
+		if (factory == NULL) {
+			g_error ("Cannot create factory");
+			exit (1);
+		}
+
+		bonobo_running_context_auto_exit_unref (BONOBO_OBJECT (factory));
+		bonobo_main ();
 	}
 
-	bonobo_running_context_auto_exit_unref (BONOBO_OBJECT (factory));
-
 	return FALSE;
 }
 
@@ -148,10 +159,14 @@ main (int argc,
 	gnome_program_init ("CDDBSlave2", VERSION, LIBGNOMEUI_MODULE,
 			    argc, argv, NULL);
 
+	if (g_getenv ("CDDB_SLAVE_DEBUG")) {
+		cddb_debugging = TRUE;
+		g_print ("CDDBSlave2 debugging turned on.\n");
+	}
 	cddbdir = g_build_filename (g_get_home_dir (),
 				    ".cddbslave", NULL);
 	if (g_file_test (cddbdir, G_FILE_TEST_EXISTS) == FALSE) {
-		mkdir (cddbdir, 0775);
+		mkdir (cddbdir, 0755);
 	}
 
 	if (g_file_test (cddbdir, G_FILE_TEST_IS_DIR) == FALSE) {
@@ -159,8 +174,7 @@ main (int argc,
 	}
 	g_free (cddbdir);
 	
-	g_idle_add (cddbslave_init, NULL);
-	bonobo_main ();
+	cddbslave_init (NULL);
 
 	exit (0);
 }
